#
# Terran Land-Based Cruise Missile
#
local TMissileCruiseProjectile = import('/lua/terranprojectiles.lua').TMissileCruiseProjectile
local Explosion = import('/lua/defaultexplosions.lua')
local EffectTemplate = import('/lua/EffectTemplates.lua')

ch_wpn_missile01 = Class(TMissileCruiseProjectile) {

    FxTrails = EffectTemplate.TMissileExhaust01,
	
    OnCreate = function(self)
        TMissileCruiseProjectile.OnCreate(self)
        self:SetCollisionShape('Sphere', 0, 0, 0, 2.0)
        self.MovementTurnLevel = 1
        self:ForkThread( self.MovementThread )
		self:ForkThread(self.SplitThread)
    end,
	
	PassDamageData = function(self, damageData)
        TMissileCruiseProjectile.PassDamageData(self,damageData)
        local launcherbp = self:GetLauncher():GetBlueprint()  
        self.ChildDamageData = table.copy(self.DamageData)
        self.ChildDamageData.DamageAmount = launcherbp.SplitDamage.DamageAmount or 1
        self.ChildDamageData.DamageRadius = launcherbp.SplitDamage.DamageRadius or 0   
    end,

    MovementThread = function(self)
        local army = self:GetArmy()
        self:TrackTarget(false)
        self:SetCollision(true)
        self:SetTurnRate(5)
        WaitSeconds(0.5)
        self:TrackTarget(true)
        self:SetTurnRate(10)
        WaitSeconds(0.5)
        while not self:BeenDestroyed() do
            self:SetTurnRateByDist()
            WaitSeconds(0.5)
        end
    end,

    SetTurnRateByDist = function(self)
        local dist = self:GetDistanceToTarget()
        if dist > 125 and self.MovementTurnLevel < 2 then
            self:SetTurnRate(30)
            self.MovementTurnLevel = 1
        elseif dist > 85 and dist <= 125 and self.MovementTurnLevel < 3 then
            self:SetTurnRate(40)
            self.MovementTurnLevel = 2
        elseif dist > 40 and dist <= 85 and self.MovementTurnLevel < 4 then
            self:SetTurnRate(50)
            self.MovementTurnLevel = 3
        elseif dist < 40 then
            self:SetTurnRate(85)
            self.MovementTurnLevel = 4
        end
    end,

    GetDistanceToTarget = function(self)
        local tpos = self:GetCurrentTargetPosition()
        local mpos = self:GetPosition()
        local dist = VDist2(mpos[1], mpos[3], tpos[1], tpos[3])
        return dist
    end,
	
	SplitThread = function(self)
		WaitSeconds(2)
		
		self.Split = true        
		local vx, vy, vz = self:GetVelocity()
		local velocity = 6
		local ChildProjectileBPSmallMissile = '/mods/Turret Compilation/projectiles/ch_wpn_missile03/ch_wpn_missile03_proj.bp'
		local ChildProjectileBPMissileEmpty = '/mods/Turret Compilation/projectiles/ch_wpn_missile02/ch_wpn_missile02_proj.bp'
		local spawnX = 1
		
		local xVec = vx + spawnX
		local yVec = vy
		local zVec = vz
		local proj = self:CreateChildProjectile(ChildProjectileBPSmallMissile)
		proj:SetVelocity(xVec,yVec,zVec)
		proj:SetVelocity(velocity)
		proj:PassDamageData(self.ChildDamageData)

		xVec = vx - spawnX
		yVec = vy
		zVec = vz
		proj = self:CreateChildProjectile(ChildProjectileBPSmallMissile)
		proj:SetVelocity(xVec,yVec,zVec)
		proj:SetVelocity(velocity)
		proj:PassDamageData(self.ChildDamageData)
		
		xVec = vx * 0.15
		yVec = vy * 0.15
		zVec = vz * 0.15
		proj = self:CreateChildProjectile(ChildProjectileBPMissileEmpty)
		proj:SetVelocity(xVec,yVec,zVec)
		proj:SetVelocity(velocity)
		
		self:Destroy()
    end,
}
TypeClass = ch_wpn_missile01