local TStructureUnit = import('/lua/terranunits.lua').TStructureUnit
local TIFHighBallisticMortarWeapon = import('/lua/terranweapons.lua').TIFHighBallisticMortarWeapon
#local ADFTractorClaw = import('/lua/aeonweapons.lua').ADFTractorClaw
local DefaultBeamWeapon = import('/lua/sim/DefaultWeapons.lua').DefaultBeamWeapon
local TractorClawCollisionBeam = import('/lua/defaultcollisionbeams.lua').TractorClawCollisionBeam

ADFTractorClaw = Class(DefaultBeamWeapon) {
    BeamType = TractorClawCollisionBeam,
    FxMuzzleFlash = {},
	
    PlayFxBeamStart = function(self, muzzle)
		#local unit = self:GetLauncher()
        local target = self:GetCurrentTarget()
        if not target or
            EntityCategoryContains(categories.STRUCTURE, target) or
            EntityCategoryContains(categories.COMMAND, target) or
            EntityCategoryContains(categories.EXPERIMENTAL, target) or
            EntityCategoryContains(categories.NAVAL, target) or
            EntityCategoryContains(categories.SUBCOMMANDER, target) or
            not EntityCategoryContains(categories.ALLUNITS, target) then
            return
        end
        DefaultBeamWeapon.PlayFxBeamStart(self, muzzle)

        self.TT1 = self:ForkThread(self.TractorThread, target)
        self:ForkThread(self.TractorWatchThread, target)
    end,

    OnLostTarget = function(self)
        self:AimManipulatorSetEnabled(true)
        DefaultBeamWeapon.OnLostTarget(self)
    end,

    TractorThread = function(self, target)
        self.unit.Trash:Add(target)
        local beam = self.Beams[1].Beam
        if not beam then return end


        local muzzle = self:GetBlueprint().MuzzleSpecial
        if not muzzle then return end


        local pos0 = beam:GetPosition(0)
        local pos1 = beam:GetPosition(1)

        local dist = VDist3(pos0, pos1)

        self.Slider = CreateSlider(self.unit, muzzle, 0, 0, dist, -1, true)

        WaitFor(self.Slider)

        target:AttachBoneTo(-1, self.unit, muzzle)

        self.AimControl:SetResetPoseTime(10)
        target:SetDoNotTarget(true)

        self.Slider:SetSpeed(15)
        self.Slider:SetGoal(0,0,0)

        WaitFor(self.Slider)

        if not target:IsDead() then
            target:Destroy()
        end
        self.AimControl:SetResetPoseTime(2)
    end,

    TractorWatchThread = function(self, target)
		local targetID = target:GetUnitId()
        while not target:IsDead() do
            WaitTicks(1)
        end
		
		local displayName = self:GetBlueprint().DisplayName
		if displayName == "Tractor Beam 1" then
			self.unit.AmmoTractor1 = targetID
		elseif displayName == "Tractor Beam 2" then
			self.unit.AmmoTractor2 = targetID
		elseif displayName == "Tractor Beam 3" then
			self.unit.AmmoTractor3 = targetID
		elseif displayName == "Tractor Beam 4" then
			self.unit.AmmoTractor4 = targetID
		end
		
		self:SetEnabled(false)
        KillThread(self.TT1)
        self.TT1 = nil
        if self.Slider then
            self.Slider:Destroy()
            self.Slider = nil
        end
        self.unit:DetachAll(self:GetBlueprint().MuzzleSpecial or 0)
        #self:ResetTarget()
        self.AimControl:SetResetPoseTime(2)
    end,
}

ch_spl04 = Class(TStructureUnit) {
    Weapons = {
        MainGun = Class(TIFHighBallisticMortarWeapon) {
			PlayFxMuzzleSequence = function(self, muzzle)
				TIFHighBallisticMortarWeapon.PlayFxMuzzleSequence(self, muzzle)
				local army = self.unit:GetArmy()
				local unit = self.unit
				
				if unit.AmmoHas == 1 then
					unit:GetWeapon(2):ResetTarget()
					unit.tractorAmmo01:Destroy()
					unit.tractorAmmo01 = nil
				elseif unit.AmmoHas == 2 then
					unit:GetWeapon(3):ResetTarget()
					unit.tractorAmmo02:Destroy()
					unit.tractorAmmo02 = nil
				elseif unit.AmmoHas == 3 then
					unit:GetWeapon(4):ResetTarget()
					unit.tractorAmmo03:Destroy()
					unit.tractorAmmo03 = nil
				elseif unit.AmmoHas == 4 then
					unit:GetWeapon(5):ResetTarget()
					unit.tractorAmmo04:Destroy()
					unit.tractorAmmo04 = nil
				end
				unit.AmmoHas = 0

				local BeamThread = ForkThread(function(self)
					local exhaustLeftTop = CreateAttachedEmitter(unit, 'Exhaust_Left_Top', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.3)
					local exhaustLeftMiddle = CreateAttachedEmitter(unit, 'Exhaust_Left_Middle', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					local exhaustLeftBottom = CreateAttachedEmitter(unit, 'Exhaust_Left_Bottom', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					local exhaustMiddleTop = CreateAttachedEmitter(unit, 'Exhaust_Middle_Top', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.3)
					local exhaustMiddleMiddle = CreateAttachedEmitter(unit, 'Exhaust_Middle_Middle', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					local exhaustMiddleBottom = CreateAttachedEmitter(unit, 'Exhaust_Middle_Bottom', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					local exhaustRightTop = CreateAttachedEmitter(unit, 'Exhaust_Right_Top', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.3)
					local exhaustRightMiddle = CreateAttachedEmitter(unit, 'Exhaust_Right_Middle', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					local exhaustRightBottom = CreateAttachedEmitter(unit, 'Exhaust_Right_Bottom', army, '/effects/emitters/dirty_exhaust_smoke_01_emit.bp'):ScaleEmitter(0.2)
					WaitSeconds(0.7)
					exhaustLeftTop:Destroy()
					exhaustLeftMiddle:Destroy()
					exhaustLeftBottom:Destroy()
					exhaustMiddleTop:Destroy()
					exhaustMiddleMiddle:Destroy()
					exhaustMiddleBottom:Destroy()
					exhaustRightTop:Destroy()
					exhaustRightMiddle:Destroy()
					exhaustRightBottom:Destroy()
					KillThread(self)
				end)
			end,
		},
		TractorBeam1 = Class(ADFTractorClaw) {},
		TractorBeam2 = Class(ADFTractorClaw) {},
		TractorBeam3 = Class(ADFTractorClaw) {},
		TractorBeam4 = Class(ADFTractorClaw) {},
    },
	
	OnCreate = function(self)
		TStructureUnit.OnCreate(self)
		#self:GetWeapon(1):ChangeProjectileBlueprint('/projectiles/TIFArtillery01/TIFArtillery01_proj.bp')
		#self:GetWeapon(1):GetProjectile().GetBlueprint().Display.Mesh.LODs.MeshName = '/units/url0202/url0202_LOD0.scm'
		#LOG (self:GetWeapon(1):GetBlueprint())
		#self:GetWeapon(1):ChangeProjectileBlueprint('/mods/Turret Compilation/projectiles/ch_wpn_unitLauncher_url0101/ch_wpn_unitLauncher_url0101_proj.bp')
		local unit = self
		local army = self:GetArmy()
		self.AmmoTractor1 = nil
		self.AmmoTractor2 = nil
		self.AmmoTractor3 = nil
		self.AmmoTractor4 = nil
		self.AmmoHas = 0
		self.AmmoUnit = nil
		self.tractorAmmo01 = nil
		self.tractorAmmo02 = nil
		self.tractorAmmo03 = nil
		self.tractorAmmo04 = nil
		
		local AmmoCheckThread = ForkThread(function(self)
			while not unit:IsDead() do
				local ammoUnit = nil
				if not (unit.AmmoTractor1 == nil) then
					if unit.tractorAmmo01 == nil then
						unit.tractorAmmo01 = CreateAttachedEmitter(unit, 'Tractor01', army, '/effects/emitters/quantum_generator_01_emit.bp'):ScaleEmitter(0.1)
					end
					if unit.AmmoHas == 0 then
						unit.AmmoHas = 1
						ammoUnit = unit.AmmoTractor1
						unit.AmmoTractor1 = nil
					end
				elseif not (unit.AmmoTractor2 == nil) then
					if unit.tractorAmmo02 == nil then
						unit.tractorAmmo02 = CreateAttachedEmitter(unit, 'Tractor02', army, '/effects/emitters/quantum_generator_01_emit.bp'):ScaleEmitter(0.1)
					end
					if unit.AmmoHas == 0 then
						unit.AmmoHas = 2
						ammoUnit = unit.AmmoTractor2
						unit.AmmoTractor2 = nil
					end
				elseif not (unit.AmmoTractor3 == nil) then
					if unit.tractorAmmo03 == nil then
						unit.tractorAmmo03 = CreateAttachedEmitter(unit, 'Tractor03', army, '/effects/emitters/quantum_generator_01_emit.bp'):ScaleEmitter(0.1)
					end
					if unit.AmmoHas == 0 then
						unit.AmmoHas = 3
						ammoUnit = unit.AmmoTractor3
						unit.AmmoTractor3 = nil
					end
				elseif not (unit.AmmoTractor4 == nil) then
					if unit.tractorAmmo04 == nil then
						unit.tractorAmmo04 = CreateAttachedEmitter(unit, 'Tractor04', army, '/effects/emitters/quantum_generator_01_emit.bp'):ScaleEmitter(0.1)
					end
					if unit.AmmoHas == 0 then
						unit.AmmoHas = 4
						ammoUnit = unit.AmmoTractor4
						unit.AmmoTractor4 = nil
					end
				end
				
				if not (ammoUnit == nil) then
					unit:GetWeapon(1):ChangeProjectileBlueprint('/mods/Turret Compilation/projectiles/ch_wpn_unitLauncher_' .. ammoUnit .. '/ch_wpn_unitLauncher_' .. ammoUnit .. '_proj.bp')
				end
				
				if not (unit.AmmoHas == 0) then
					unit:GetWeapon(1):SetEnabled(true)
				else
					unit:GetWeapon(1):SetEnabled(false)
				end
				WaitSeconds(1)
			end
			KillThread(self)
		end)
    end,
}

TypeClass = ch_spl04